//
//  ViewController.swift
//  RadioAppDemo
//
//  Created by Andres Käver on 17/05/2019.
//  Copyright © 2019 TalTech IT College - Andres Käver. All rights reserved.
//

import UIKit
import AVFoundation
import MediaPlayer

class ViewController: UIViewController {
    enum AudioStatus {
        case stopped
        case buffering
        case playing
    }

    var audioStatus = AudioStatus.stopped
    var audioPlayer: AVPlayer!
    
    @IBOutlet weak var controlButton: UIButton!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try AVAudioSession.sharedInstance().setCategory(AVAudioSession.Category(rawValue: "AVAudioSessionCategoryPlayback"))
            try AVAudioSession.sharedInstance().setActive(true)
        } catch {
            print("Error in audiosession! \(error)")
        }
    }


    @IBAction func controlButtonTouchUpInside(_ sender: UIButton) {
        switch audioStatus {
        case .buffering, .playing:
            audioStop()
        case .stopped:
            audioPrepareForPlaying()
        }
    }
    
    func audioPrepareForPlaying(){
        controlButton.setTitle("Buffering", for: UIControl.State.normal)
        audioStatus = .buffering
        // it starts to prepare it immediately
        audioPlayer = AVPlayer(url: URL(string: "http://skymedia.babahhcdn.com/rrap")!)
        //NSKeyValueObservingOptions()
        // add key-value-observer to get audioplayer events - like preparation is done
        audioPlayer.addObserver(self, forKeyPath: "status", options: [], context: nil)
    }
    
    func audioStop(){
        audioStatus = .stopped
        controlButton.setTitle("PLAY!", for: UIControl.State.normal)
        if (audioPlayer != nil){
            audioPlayer.pause()
            audioPlayer.removeObserver(self, forKeyPath: "status")
            audioPlayer = nil
        }
    }
    
    func audioPlay(){
        audioStatus = .playing
        controlButton.setTitle("STOP!", for: UIControl.State.normal)
        audioPlayer.play()
    }
    
    override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {

        // who was the sender, is for as
        if object is AVPlayer {
            switch audioPlayer.status {
            case .failed:
                print("audioplayer failed!")
            case .readyToPlay:
                print("audioplayer readyToPlay!")
                audioPlay()
            case .unknown:
                print("audioplayer unknown status!")
            @unknown default:
                fatalError("")
            }
        }
        
    }
}

